/*
 * security/tomoyo/audit.c
 *
 * Audit functions for TOMOYO Linux
 *
 * Copyright (C) 2005-2007  NTT DATA CORPORATION
 *
 * Version: 2.0   2007/06/05
 */

#include "tomoyo.h"
#include <linux/audit.h>
#include <linux/version.h>

/* move to include/linux/audit.h */
#define AUDIT_TOMOYO 2001

char *tomoyo_init_audit_log(int *len)
{
	char *buf;
	struct timeval tv;
	struct task_struct *task = current;
	const char *domainname =
		((struct tomoyo_security *) current->security)->domain_info->domainname->name;
	do_gettimeofday(&tv);
	*len += strlen(domainname) + 256;
	if ((buf = tomoyo_alloc(*len)) != NULL)
		snprintf(buf,
		         (*len) - 1,
		         "#timestamp=%lu pid=%d uid=%d gid=%d euid=%d egid=%d "
		         "suid=%d sgid=%d fsuid=%d fsgid=%d\n%s\n",
		         tv.tv_sec, task->pid, task->uid, task->gid, task->euid, task->egid,
		         task->suid, task->sgid, task->fsuid, task->fsgid, domainname);
	return buf;
}

/*
 * Write audit log.
 * Caller must allocate buf with tomoyo_init_audit_log().
 */
int tomoyo_write_audit_log(char *buf, const int is_granted)
{
	struct audit_buffer *ab;
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,12)
	ab = audit_log_start(current->audit_context);
#elif  LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,13)
	ab = audit_log_start(current->audit_context, GFP_KERNEL);
#else
	ab = audit_log_start(current->audit_context, GFP_KERNEL, AUDIT_TOMOYO);
#endif
	if (ab) {
		audit_log_format(ab,
		                 "%s\n%s",
		                 is_granted ? "TOMOYO_GRANT_LOG" : "TOMOYO_REJECT_LOG", buf);
		audit_log_end(ab);
	}
	tomoyo_free(buf);
	return ab ? 0 : -ENOMEM;
}
