/*
 * security/tomoyo/include/realpath.h
 *
 * Get the canonicalized absolute pathnames. The basis for TOMOYO.
 *
 * Copyright (C) 2005-2007  NTT DATA CORPORATION
 *
 * Version: 2.0   2007/06/05
 *
 */

#ifndef _TOMOYO_REALPATH_H
#define _TOMOYO_REALPATH_H

struct path_info;

/* Returns realpath(3) of the given pathname but ignores chroot'ed root. */
int tomoyo_realpath_from_dentry2(struct dentry *dentry,
                                 struct vfsmount *mnt,
                                 char *newname,
                                 int newname_len);

/* Returns realpath(3) of the given pathname but ignores chroot'ed root. */
/* These functions use tomoyo_alloc(), so caller must tomoyo_free() */
/* if these functions didn't return NULL. */
char *tomoyo_realpath(const char *pathname);
char *tomoyo_realpath_nofollow(const char *pathname);
char *tomoyo_realpath_from_dentry(struct dentry *dentry, struct vfsmount *mnt);

/* Allocate memory for structures. */
/* The RAM is chunked, so NEVER try to kfree() the returned pointer. */
void *tomoyo_alloc_element(const unsigned int size);

/* Get used RAM size for tomoyo_alloc_elements(). */
unsigned int tomoyo_get_memory_used_for_elements(void);

/* Keep the given name on the RAM. */
/* The RAM is shared, so NEVER try to modify or kfree() the returned name. */
const struct path_info *tomoyo_save_name(const char *name);

/* Get used RAM size for tomoyo_save_name(). */
unsigned int tomoyo_get_memory_used_for_save_name(void);

unsigned int tomoyo_get_memory_used_for_dynamic(void);

#endif
